#include	<windows.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<gl\gl.h>										// Header File For The OpenGL32 Library
#include	<gl\glu.h>										// Header File For The GLu32 Library

#include	"car.h"

CAR_Header_t	*carfile;

int				modeltris;
int				modelverts;
char			modelpath[260];
char			modelname[260];
float			rotate = 225;

extern	HWND	Window;
extern	BOOL	Part_Body;
extern	BOOL	Part_FLW;
extern	BOOL	Part_FRW;
extern	BOOL	Part_RLW;
extern	BOOL	Part_RRW;
extern	BOOL	model_open;

extern	float	frame_time;
extern	char	windowtext[128];

BOOL			LoadTexture (void);

BOOL LoadCar (void)
{
	FILE	*fin;

	fin = fopen (modelpath, "rb");

	if (!fin)
	{
		return FALSE;
	}

	carfile	= malloc(sizeof(CAR_Header_t));
	memset (carfile, 0, sizeof(CAR_Header_t));

	fread (carfile, 1, sizeof(CAR_Header_t), fin);

	fclose(fin);

	modeltris	= carfile->Body.NumTriangles +
				  carfile->Wheel[0].NumTriangles +
				  carfile->Wheel[1].NumTriangles +
				  carfile->Wheel[2].NumTriangles +
				  carfile->Wheel[3].NumTriangles;

	modelverts	= carfile->Body.NumVertices +
				  carfile->Wheel[0].NumVertices +
				  carfile->Wheel[1].NumVertices +
				  carfile->Wheel[2].NumVertices +
				  carfile->Wheel[3].NumVertices;

	sprintf (windowtext, "MRViewer - %s     %i tris %i verts", modelname, modeltris, modelverts);
	SetWindowText (Window, windowtext);

	return TRUE;
}

BOOL OpenCar (void)
{
	static	OPENFILENAME	ofn;
	static	char			file[260];
	static	char			filetitle[260];
	static	char			dir[260];

	GetCurrentDirectory (260, dir);

	file[0]				= '\0';
	filetitle[0]		= '\0';

	ofn.lStructSize		= sizeof(OPENFILENAME); 
	ofn.hwndOwner		= Window;
	ofn.lpstrFilter		= "Miniracer car file (*.car)\0*.car\0"; 
	ofn.nFilterIndex	= 1; 
	ofn.lpstrFile		= file; 
	ofn.nMaxFile		= 260; 
	ofn.lpstrTitle		= "Open Model\0";
	ofn.lpstrFileTitle	= filetitle; 
	ofn.nMaxFileTitle	= 260; 
	ofn.lpstrInitialDir	= dir; 
	ofn.Flags			= OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;

	strcpy (modelpath, "none");
	strcpy (modelname, "none");

	if (GetOpenFileName (&ofn))
	{
		strcpy (modelpath, file);
		strcpy (modelname, filetitle);
	
		if (LoadCar ())
		{
			LoadTexture ();
			return 1;
		}
	}

	return 0;
}

void DrawWireBody (CAR_Part_Body_t *Part)
{
	int	i;

	glPushMatrix();
	glTranslatef (Part->Origin[0], Part->Origin[1], Part->Origin[2]);

	glVertexPointer (3, GL_FLOAT, sizeof(float[3]), Part->Verts);
	glNormalPointer (   GL_FLOAT, sizeof(float[3]), Part->Norms);

	for (i=0; i<Part->NumTriangles; i++)
	{
		glBegin(GL_LINE_LOOP);

		glTexCoord2f  (Part->Tris[i].U[0], Part->Tris[i].V[0]);
		glArrayElement(Part->Tris[i].I[0]);

		glTexCoord2f  (Part->Tris[i].U[1], Part->Tris[i].V[1]);
		glArrayElement(Part->Tris[i].I[1]);

		glTexCoord2f  (Part->Tris[i].U[2], Part->Tris[i].V[2]);
		glArrayElement(Part->Tris[i].I[2]);

		glEnd();
	}

	glPopMatrix();
}

void DrawWireWheel (CAR_Part_Wheel_t *Part)
{
	int	i;

	glPushMatrix();
	glTranslatef (Part->Origin[0], Part->Origin[1], Part->Origin[2]);

	glVertexPointer (3, GL_FLOAT, sizeof(float[3]), Part->Verts);
	glNormalPointer (   GL_FLOAT, sizeof(float[3]), Part->Norms);

	for (i=0; i<Part->NumTriangles; i++)
	{
		glBegin(GL_LINE_LOOP);

		glTexCoord2f  (Part->Tris[i].U[0], Part->Tris[i].V[0]);
		glArrayElement(Part->Tris[i].I[0]);

		glTexCoord2f  (Part->Tris[i].U[1], Part->Tris[i].V[1]);
		glArrayElement(Part->Tris[i].I[1]);

		glTexCoord2f  (Part->Tris[i].U[2], Part->Tris[i].V[2]);
		glArrayElement(Part->Tris[i].I[2]);

		glEnd();
	}

	glPopMatrix();
}

void DrawPointBody (CAR_Part_Body_t *Part)
{
	int	i;

	glPushMatrix();
	glTranslatef (Part->Origin[0], Part->Origin[1], Part->Origin[2]);

	glBegin(GL_POINTS);

	for (i=0; i<Part->NumVertices; i++)
	{
		glVertex3fv (Part->Verts[i]);
		glNormal3fv (Part->Norms[i]);
	}

	glEnd();

	glPopMatrix();
}

void DrawPointWheel (CAR_Part_Wheel_t *Part)
{
	int	i;

	glPushMatrix();
	glTranslatef (Part->Origin[0], Part->Origin[1], Part->Origin[2]);

	glBegin(GL_POINTS);

	for (i=0; i<Part->NumVertices; i++)
		glVertex3fv (Part->Verts[i]);

	glEnd();

	glPopMatrix();
}

void DrawBody (CAR_Part_Body_t *Part)
{
	int	i;

	glPushMatrix();
	glTranslatef (Part->Origin[0], Part->Origin[1], Part->Origin[2]);

	glVertexPointer (3, GL_FLOAT, sizeof(float[3]), Part->Verts);
	glNormalPointer (   GL_FLOAT, sizeof(float[3]), Part->Norms);

	glBegin(GL_TRIANGLES);
	for (i=0; i<Part->NumTriangles; i++)
	{
		glTexCoord2f  (Part->Tris[i].U[0], Part->Tris[i].V[0]);
		glArrayElement(Part->Tris[i].I[0]);

		glTexCoord2f  (Part->Tris[i].U[1], Part->Tris[i].V[1]);
		glArrayElement(Part->Tris[i].I[1]);

		glTexCoord2f  (Part->Tris[i].U[2], Part->Tris[i].V[2]);
		glArrayElement(Part->Tris[i].I[2]);
	}
	glEnd();

	glPopMatrix();
}

void DrawWheel (CAR_Part_Wheel_t *Part)
{
	int	i;

	glPushMatrix();
	glTranslatef (Part->Origin[0], Part->Origin[1], Part->Origin[2]);

	glVertexPointer (3, GL_FLOAT, sizeof(float[3]), Part->Verts);
	glNormalPointer (   GL_FLOAT, sizeof(float[3]), Part->Norms);

	glBegin(GL_TRIANGLES);
	for (i=0; i<Part->NumTriangles; i++)
	{
		glTexCoord2f  (Part->Tris[i].U[0], Part->Tris[i].V[0]);
		glArrayElement(Part->Tris[i].I[0]);

		glTexCoord2f  (Part->Tris[i].U[1], Part->Tris[i].V[1]);
		glArrayElement(Part->Tris[i].I[1]);

		glTexCoord2f  (Part->Tris[i].U[2], Part->Tris[i].V[2]);
		glArrayElement(Part->Tris[i].I[2]);
	}
	glEnd();

	glPopMatrix();
}

void DrawCar (BOOL Rotate, BOOL WireFrame, BOOL Points, BOOL Solid)
{
	if (Rotate)
		rotate -= 180*frame_time;

	if (rotate < 0)
		rotate += 360;

	glTranslatef(0.0f,0.0f,-40.0f);

	glRotatef(-75, 1, 0, 0);

	glRotatef(rotate, 0, 0, 1);

	glEnableClientState (GL_VERTEX_ARRAY);
	glEnableClientState (GL_NORMAL_ARRAY);

	if (Solid)
	{
		if (Part_Body)	DrawBody  (&carfile->Body);
		if (Part_FLW)	DrawWheel (&carfile->Wheel[0]);
		if (Part_FRW)	DrawWheel (&carfile->Wheel[1]);
		if (Part_RLW)	DrawWheel (&carfile->Wheel[2]);
		if (Part_RRW)	DrawWheel (&carfile->Wheel[3]);
	}

	if (WireFrame)
	{
		if (Part_Body)	DrawWireBody  (&carfile->Body);
		if (Part_FLW)	DrawWireWheel (&carfile->Wheel[0]);
		if (Part_FRW)	DrawWireWheel (&carfile->Wheel[1]);
		if (Part_RLW)	DrawWireWheel (&carfile->Wheel[2]);
		if (Part_RRW)	DrawWireWheel (&carfile->Wheel[3]);
	}

	if (Points)
	{
		if (Part_Body)	DrawPointBody  (&carfile->Body);
		if (Part_FLW)	DrawPointWheel (&carfile->Wheel[0]);
		if (Part_FRW)	DrawPointWheel (&carfile->Wheel[1]);
		if (Part_RLW)	DrawPointWheel (&carfile->Wheel[2]);
		if (Part_RRW)	DrawPointWheel (&carfile->Wheel[3]);
	}

	glDisableClientState (GL_NORMAL_ARRAY);
	glDisableClientState (GL_VERTEX_ARRAY);
}