#include	<windows.h>
#include	<mmsystem.h>
#include	<commctrl.h>
#include	<gl\gl.h>
#include	<gl\glu.h>
#include	"resource.h"
#include	"car.h"

extern	int				modeltris;
extern	int				modelverts;
extern	char			modelpath[260];
extern	char			modelname[260];
extern	CAR_Header_t	*carfile;

HDC			baseDC		= NULL;
BOOL		done		= FALSE;
BOOL		model_open	= FALSE;	
BOOL		ReadyToDraw = FALSE;
BOOL		WireFrame	= FALSE;
BOOL		Points		= FALSE;
BOOL		Rotate		= FALSE;
BOOL		Light		= FALSE;
BOOL		Solid		= TRUE;
BOOL		Texture		= TRUE;
BOOL		Part_Body	= TRUE;
BOOL		Part_FLW	= TRUE;
BOOL		Part_FRW	= TRUE;
BOOL		Part_RLW	= TRUE;
BOOL		Part_RRW	= TRUE;
HWND		Window		= NULL;
HWND		PartList	= NULL;
HWND		VertList	= NULL;
HWND		VertEditor	= NULL;
HMENU		MainMenu	= NULL;
HGLRC		baseRC		= NULL;
HINSTANCE	hInstance	= NULL;

float		LightAmbient[4]		= {0,0,0,0};
float		LightDiffuse[4]		= {2,2,2,2};
float		LightPosition[4]	= {0,0,0,1};

float		forward				= 0;
float		left				= 0;
float		up					= 0;

float		pitch				= 0;
float		yaw					= 0;
float		roll				= 0;

float		frame_time			= 0;

int			frames				= 0;
int			new_time			= 0;
int			old_time			= 0;

int			Part_Select			= 99999;
int			Vert_Select			= 99999;
int			Part_Select_Old		= 99999;
int			Vert_Select_Old		= 99999;

char		windowtext[128];

BOOL		OpenCar (void);
void		DrawCar (BOOL Rotate, BOOL WireFrame, BOOL Points, BOOL Solid);

LRESULT		CALLBACK WindowProc (HWND, UINT, WPARAM, LPARAM);

void KillGLWindow (void)
{
	if (baseRC)
	{
		if (!wglMakeCurrent (NULL, NULL))
			MessageBox(NULL, "Release Of DC And RC Failed.", "SHUTDOWN ERROR", MB_OK | MB_ICONINFORMATION);

		if (!wglDeleteContext (baseRC))
			MessageBox (NULL, "Release Rendering Context Failed.", "SHUTDOWN ERROR", MB_OK | MB_ICONINFORMATION);
	}

	if (baseDC && !ReleaseDC (Window, baseDC))
		MessageBox (NULL, "Release Of Device Context Failed.", "SHUTDOWN ERROR", MB_OK | MB_ICONINFORMATION);

	if (Window && !DestroyWindow (Window))
		MessageBox (NULL, "Could Not Release Window.", "SHUTDOWN ERROR", MB_OK | MB_ICONINFORMATION);

	if (!UnregisterClass ("MRViewer", hInstance))
		MessageBox (NULL, "Could Not Unregister Class.", "SHUTDOWN ERROR", MB_OK | MB_ICONINFORMATION);

	baseRC		= NULL;
	baseDC		= NULL;
	Window		= NULL;
	hInstance	= NULL;
}

void ReSizeGLScene (int width, int height)
{
	if (height == 0)
		height = 1;

	glViewport(0,0,width,height);

	glMatrixMode (GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(45.0f,(float)width/(float)height,0.1f,1000.0f);

	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity();
}

void GLInit (void)
{
	glHint			(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glEnable		(GL_LIGHT0);
	glEnable		(GL_CULL_FACE);
	glEnable		(GL_DEPTH_TEST);
	glEnable		(GL_TEXTURE_2D);	
	glLightfv		(GL_LIGHT0, GL_AMBIENT,  LightAmbient);
	glLightfv		(GL_LIGHT0, GL_DIFFUSE,  LightDiffuse);
	glLightfv		(GL_LIGHT0, GL_POSITION, LightPosition);
	glCullFace		(GL_FRONT);
	glDepthFunc		(GL_LEQUAL);
	glClearDepth	(1);
	glClearColor	(0,0,0,0);
	glShadeModel	(GL_SMOOTH);
	glPolygonMode	(GL_FRONT_AND_BACK, GL_FILL);
	glLightModeli	(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
}

int DrawGLScene (void)
{
	frames++;

	frame_time = (float)new_time;

	new_time = timeGetTime ();

	frame_time = (new_time - frame_time) / 1000;

	if (new_time > old_time)
	{
		char	buf[256];

		sprintf (buf, "%s     %i FPS", windowtext, frames);
		SetWindowText (Window, buf);

		old_time = new_time + 1000;
		frames = 0;
	}
	
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity ();

	glPushMatrix ();

	glTranslatef (left, up ,forward);

	DrawCar (Rotate, WireFrame, Points, Solid);

	glPopMatrix();

	return TRUE;
}

void CenterWindow (int width, int height)
{
	int     CenterX, CenterY;

	CenterX = (int)((GetSystemMetrics(SM_CXSCREEN) - width)  * 0.5);
	CenterY = (int)((GetSystemMetrics(SM_CYSCREEN) - height) * 0.5);

	SetWindowPos (Window, NULL, CenterX, CenterY, 0, 0,	SWP_NOSIZE | SWP_NOZORDER | SWP_SHOWWINDOW | SWP_DRAWFRAME);
}

BOOL CreateGLWindow (int width, int height)
{
	int						PixelFormat;
	DWORD					WindowStyle;
	WNDCLASS				wc;
	PIXELFORMATDESCRIPTOR	pfd;

	memset (&wc,  0, sizeof(wc));
	memset (&pfd, 0, sizeof(pfd));

	hInstance			= GetModuleHandle (NULL);
	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wc.lpfnWndProc		= (WNDPROC)WindowProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= hInstance;
	wc.hIcon			= LoadIcon   (NULL, IDI_WINLOGO);
	wc.hCursor			= LoadCursor (NULL, IDC_ARROW);
	wc.hbrBackground	= NULL;
	wc.lpszMenuName		= MAKEINTRESOURCE (IDR_MENU1);
	wc.lpszClassName	= "MRViewer";

	if (!RegisterClass (&wc))
	{
		MessageBox (NULL, "Failed To Register The Window Class.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}
	
	WindowStyle		= WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

	if (!(Window = CreateWindow
		("MRViewer",
		 "MRViewer",
		 WindowStyle,
		 0,
		 0,
		 width,
		 height,
		 NULL,
		 NULL,
		 hInstance,
		 NULL)))
	{
		MessageBox (NULL, "Window Creation Error.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		KillGLWindow ();
		return FALSE;
	}

	CenterWindow (width, height);

	pfd.nSize			= sizeof(PIXELFORMATDESCRIPTOR);
	pfd.nVersion		= 1;
	pfd.dwFlags			= PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
	pfd.iPixelType		= PFD_TYPE_RGBA;
	pfd.cColorBits		= 16;
	pfd.cDepthBits		= 16;
	pfd.iLayerType		= PFD_MAIN_PLANE;

	if (!(baseDC = GetDC (Window)))
	{
		KillGLWindow ();
		MessageBox (NULL, "Can't Create A GL Device Context.", "ERROR", MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}

	if (!(PixelFormat = ChoosePixelFormat (baseDC, &pfd)))
	{
		KillGLWindow ();
		MessageBox (NULL, "Can't Find A Suitable PixelFormat.", "ERROR", MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}

	if(!SetPixelFormat (baseDC, PixelFormat, &pfd))
	{
		KillGLWindow ();
		MessageBox (NULL, "Can't Set The PixelFormat.", "ERROR", MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}

	if (!(baseRC = wglCreateContext (baseDC)))
	{
		KillGLWindow ();
		MessageBox (NULL, "Can't Create A GL Rendering Context.", "ERROR", MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}

	if(!wglMakeCurrent (baseDC,baseRC))
	{
		KillGLWindow ();
		MessageBox (NULL, "Can't Activate The GL Rendering Context.", "ERROR", MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}

	MainMenu = GetMenu (Window);

	CheckMenuItem (MainMenu, ID_VIEW_TEXTURE, MF_CHECKED);
	CheckMenuItem (MainMenu, ID_VIEW_SOLID,	  MF_CHECKED);
	CheckMenuItem (MainMenu, ID_PARTS_BODY,	  MF_CHECKED);
	CheckMenuItem (MainMenu, ID_PARTS_FLW,	  MF_CHECKED);
	CheckMenuItem (MainMenu, ID_PARTS_FRW,	  MF_CHECKED);
	CheckMenuItem (MainMenu, ID_PARTS_RLW,	  MF_CHECKED);
	CheckMenuItem (MainMenu, ID_PARTS_RRW,	  MF_CHECKED);

	UpdateWindow  (Window);
	ReSizeGLScene (width, height);

	GLInit ();

	ShowWindow	  (Window, SW_SHOW);

	return TRUE;
}

BOOL CALLBACK ToolsDialogProc (HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	int		i;

	switch (uMsg)
	{
		case WM_SYSCOMMAND:
		{
			if (wParam == SC_CLOSE)
				EndDialog (hWnd, 0);
			break;
		}

		case WM_INITDIALOG:
		{
			PartList = GetDlgItem (hWnd, IDC_LIST1);
			VertList = GetDlgItem (hWnd, IDC_LIST2);

			SendMessage (PartList, LB_ADDSTRING, 0, (LPARAM) (LPCTSTR) "Body");
			SendMessage (PartList, LB_ADDSTRING, 0, (LPARAM) (LPCTSTR) "Front Left Wheel");
			SendMessage (PartList, LB_ADDSTRING, 0, (LPARAM) (LPCTSTR) "Front Right Wheel");
			SendMessage (PartList, LB_ADDSTRING, 0, (LPARAM) (LPCTSTR) "Rear Left Wheel");
			SendMessage (PartList, LB_ADDSTRING, 0, (LPARAM) (LPCTSTR) "Rear Right Wheel");

			break;
		}

		case WM_COMMAND:
		{
			switch (HIWORD(wParam))
			{
			case LBN_SELCHANGE:
				Part_Select	= SendMessage (PartList, LB_GETCURSEL, 0, 0);

				if (Part_Select != Part_Select_Old)
				{
					char	part[4];
					Part_Select_Old = Part_Select;
					SendMessage (VertList, LB_RESETCONTENT, 0, 0);

					switch (Part_Select)
					{
					case 0:
						for (i=0; i<carfile->Body.NumVertices; i++)
						{
							sprintf (part, "%3i", i);
							SendMessage (VertList, LB_ADDSTRING, 0, (LPARAM) (LPCTSTR) part);
						}
						break;

					case 1:
						for (i=0; i<carfile->Wheel[0].NumVertices; i++)
						{
							sprintf (part, "%3i", i);
							SendMessage (VertList, LB_ADDSTRING, 0, (LPARAM) (LPCTSTR) part);
						}
						break;

					case 2:
						for (i=0; i<carfile->Wheel[1].NumVertices; i++)
						{
							sprintf (part, "%3i", i);
							SendMessage (VertList, LB_ADDSTRING, 0, (LPARAM) (LPCTSTR) part);
						}
						break;

					case 3:
						for (i=0; i<carfile->Wheel[2].NumVertices; i++)
						{
							sprintf (part, "%3i", i);
							SendMessage (VertList, LB_ADDSTRING, 0, (LPARAM) (LPCTSTR) part);
						}
						break;

					case 4:
						for (i=0; i<carfile->Wheel[3].NumVertices; i++)
						{
							sprintf (part, "%3i", i);
							SendMessage (VertList, LB_ADDSTRING, 0, (LPARAM) (LPCTSTR) part);
						}
						break;
					}
				}
				else
					Vert_Select	= SendMessage (VertList, LB_GETCURSEL, 0, 0);

				if (Vert_Select != Vert_Select_Old)
				{
					char	vert0[8];
					char	vert1[8];
					char	vert2[8];

					Vert_Select_Old = Vert_Select;

					switch (Part_Select)
					{
					case 0:
						sprintf (vert0, "%.3f", carfile->Body.Verts[Vert_Select][0]);
						sprintf (vert1, "%.3f", carfile->Body.Verts[Vert_Select][1]);
						sprintf (vert2, "%.3f", carfile->Body.Verts[Vert_Select][2]);
						break;

					case 1:
						sprintf (vert0, "%.3f", carfile->Wheel[0].Verts[Vert_Select][0]);
						sprintf (vert1, "%.3f", carfile->Wheel[0].Verts[Vert_Select][1]);
						sprintf (vert2, "%.3f", carfile->Wheel[0].Verts[Vert_Select][2]);
						break;

					case 2:
						sprintf (vert0, "%.3f", carfile->Wheel[1].Verts[Vert_Select][0]);
						sprintf (vert1, "%.3f", carfile->Wheel[1].Verts[Vert_Select][1]);
						sprintf (vert2, "%.3f", carfile->Wheel[1].Verts[Vert_Select][2]);
						break;

					case 3:
						sprintf (vert0, "%.3f", carfile->Wheel[2].Verts[Vert_Select][0]);
						sprintf (vert1, "%.3f", carfile->Wheel[2].Verts[Vert_Select][1]);
						sprintf (vert2, "%.3f", carfile->Wheel[2].Verts[Vert_Select][2]);
						break;

					case 4:
						sprintf (vert0, "%.3f", carfile->Wheel[3].Verts[Vert_Select][0]);
						sprintf (vert1, "%.3f", carfile->Wheel[3].Verts[Vert_Select][1]);
						sprintf (vert2, "%.3f", carfile->Wheel[3].Verts[Vert_Select][2]);
						break;
					}

					SetDlgItemText (hWnd, IDC_EDIT1, vert0);
					SetDlgItemText (hWnd, IDC_EDIT2, vert1);
					SetDlgItemText (hWnd, IDC_EDIT3, vert2);
				}
			}

			switch (LOWORD(wParam))
			{ 	
				char	verta[8];
				float	vertf[3];

				case IDC_BUTTON1:

					GetDlgItemText (hWnd, IDC_EDIT1, verta, 8);	vertf[0] = (float) atof (verta);
					GetDlgItemText (hWnd, IDC_EDIT2, verta, 8);	vertf[1] = (float) atof (verta);
					GetDlgItemText (hWnd, IDC_EDIT3, verta, 8);	vertf[2] = (float) atof (verta);

					switch (Part_Select)
					{
					case 0:
						VectorCopy (vertf, carfile->Body.Verts[Vert_Select]);
						break;

					case 1:
						VectorCopy (vertf, carfile->Wheel[0].Verts[Vert_Select]);
						break;

					case 2:
						VectorCopy (vertf, carfile->Wheel[1].Verts[Vert_Select]);
						break;

					case 3:
						VectorCopy (vertf, carfile->Wheel[2].Verts[Vert_Select]);
						break;

					case 4:
						VectorCopy (vertf, carfile->Wheel[3].Verts[Vert_Select]);
						break;
					}

					glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

					DrawGLScene ();

					SwapBuffers (baseDC);

					break;

				case IDC_BUTTON2:
					EndDialog (hWnd, 0);
					break;
			}

			break;
		}
	}

	return FALSE;
}

void CommandHandler (HWND hWnd, WPARAM wParam)
{
	switch (LOWORD (wParam))
	{
		case ID_FILES_OPEN_MODEL:

			if (model_open)
				free (carfile);

			model_open = FALSE;

			Part_Select = Part_Select_Old = Vert_Select = Vert_Select_Old = 99999;

			if (OpenCar())
				model_open = TRUE;
			break;
						
		case ID_FILES_EXIT:
			PostMessage (Window, WM_CLOSE, 0, 0);
			break;

		case ID_VIEW_WIREFRAME:
			WireFrame = !WireFrame;

			CheckMenuItem (MainMenu, ID_VIEW_WIREFRAME, WireFrame ? MF_CHECKED : MF_UNCHECKED);
			break;

		case ID_VIEW_POINTS:
			Points = !Points;

			CheckMenuItem (MainMenu, ID_VIEW_POINTS, Points ? MF_CHECKED : MF_UNCHECKED);
			break;

		case ID_VIEW_SOLID:
			Solid = !Solid;

			CheckMenuItem (MainMenu, ID_VIEW_SOLID, Solid ? MF_CHECKED : MF_UNCHECKED);
			break;

		case ID_VIEW_ROTATE:
			Rotate = !Rotate;

			CheckMenuItem (MainMenu, ID_VIEW_ROTATE, Rotate ? MF_CHECKED : MF_UNCHECKED);
			break;

		case ID_VIEW_LIGHT:
			Light = !Light;

			if (Light)
			{
				CheckMenuItem (MainMenu, ID_VIEW_LIGHT, MF_CHECKED);
				glEnable  (GL_LIGHTING);
			}
			else
			{
				CheckMenuItem (MainMenu, ID_VIEW_LIGHT, MF_UNCHECKED);
				glDisable (GL_LIGHTING);
			}
			break;

		case ID_VIEW_TEXTURE:
			Texture = !Texture;

			if (Texture)
			{
				CheckMenuItem (MainMenu, ID_VIEW_TEXTURE, MF_CHECKED);
				glEnable  (GL_TEXTURE_2D);
			}
			else
			{
				CheckMenuItem (MainMenu, ID_VIEW_TEXTURE, MF_UNCHECKED);
				glDisable (GL_TEXTURE_2D);
			}
			break;

		case ID_PARTS_BODY:
			Part_Body = !Part_Body;

			CheckMenuItem (MainMenu, ID_PARTS_BODY, Part_Body ? MF_CHECKED : MF_UNCHECKED);
			break;

		case ID_PARTS_FLW:
			Part_FLW = !Part_FLW;

			CheckMenuItem (MainMenu, ID_PARTS_FLW, Part_FLW ? MF_CHECKED : MF_UNCHECKED);
			break;

		case ID_PARTS_FRW:
			Part_FRW = !Part_FRW;

			CheckMenuItem (MainMenu, ID_PARTS_FRW, Part_FRW ? MF_CHECKED : MF_UNCHECKED);
			break;

		case ID_PARTS_RLW:
			Part_RLW = !Part_RLW;

			CheckMenuItem (MainMenu, ID_PARTS_RLW, Part_RLW ? MF_CHECKED : MF_UNCHECKED);
			break;

		case ID_PARTS_RRW:
			Part_RRW = !Part_RRW;

			CheckMenuItem (MainMenu, ID_PARTS_RRW, Part_RRW ? MF_CHECKED : MF_UNCHECKED);
			break;

		case ID_TOOLS_VERTEDITOR:
			if (model_open)
			{
				VertEditor = CreateDialog (hInstance, MAKEINTRESOURCE (IDR_VERTEDITOR), Window, (DLGPROC)ToolsDialogProc);
				
				if (VertEditor)
					ShowWindow (VertEditor, SW_SHOW);
			}
			else
				MessageBox (NULL, "No model loaded", "No model loaded", MB_OK);
			break;
	}
}

LRESULT CALLBACK WindowProc (HWND hWnd, UINT uMsg, WPARAM wParam,LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_COMMAND:
		{
			CommandHandler (hWnd, wParam);
			return 0;
		}

		case WM_CLOSE:
		case WM_DESTROY:
		{
			PostQuitMessage(0);
			return 0;
		}

		case WM_SIZE:
		{
			ReSizeGLScene (LOWORD(lParam),HIWORD(lParam));
			return 0;
		}

		case WM_KEYDOWN:
		{
			switch (wParam)
			{
				case VK_ESCAPE:
				{
					done = TRUE;
					break;
				}

				case VK_LEFT:
				{
					left++;
					break;
				}

				case VK_RIGHT:
				{
					left--;
					break;
				}

				case VK_UP:
				{
					forward++;
					break;
				}

				case VK_DOWN:
				{
					forward--;
					break;
				}
			}
			return 0;
		}

		case WM_PAINT:
		{
			if (ReadyToDraw)
			{
				glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
				if (model_open)
				{
					if (!DrawGLScene ())
						done = TRUE;
				}
				SwapBuffers (baseDC);
				ValidateRect (Window, NULL);
			}
			return 0;
		}
	}

	return DefWindowProc (hWnd, uMsg, wParam, lParam);
}

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG		Msg;

	if (!CreateGLWindow (640, 480))
		return 0;

	InitCommonControls ();

	ReadyToDraw = TRUE;

	PeekMessage (&Msg, NULL, 0, 0, PM_REMOVE);

	while (Msg.message != WM_QUIT && !done)
	{
		if (PeekMessage (&Msg, NULL, 0, 0, PM_REMOVE))
		{
			TranslateMessage (&Msg);
			DispatchMessage  (&Msg);
		}
		else
		{
			InvalidateRect (Window, NULL, FALSE);
		}
	}

	ReadyToDraw = FALSE;

	KillGLWindow ();

	if (model_open)
		free (carfile);

	return (Msg.wParam);
}